@@ -1,14 +1,43 @@
-from MAVProxy.modules.lib.mp_menu import *
-from ..lib.wx_loader import wx
-import mp_elevation
-import os
 import functools
-from mp_slipmap_util import *
+import math
+from MAVProxy.modules.mavproxy_map import mp_elevation
+import numpy as np
+import os
+import time
+
+from ..lib.wx_loader import wx
+
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipBrightness
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipCenter
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipClearLayer
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipDefaultPopup
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipFlightModeLegend
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipGrid
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipHideObject
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipIcon
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipInformation
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipKeyEvent
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipMenuEvent
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipMouseEvent
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipObject
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipObjectSelection
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipPosition
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipRemoveObject
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipThumbnail
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipZoom
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipFollow
+from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipFollowObject
+
+from MAVProxy.modules.lib import mp_util
+from MAVProxy.modules.lib import win_layout
+
+from MAVProxy.modules.lib.mp_menu import MPMenuCheckbox
+from MAVProxy.modules.lib.mp_menu import MPMenuItem
+from MAVProxy.modules.lib.mp_menu import MPMenuRadio
+from MAVProxy.modules.lib.mp_menu import MPMenuSeparator
+from MAVProxy.modules.lib.mp_menu import MPMenuSubMenu
+from MAVProxy.modules.lib.mp_menu import MPMenuTop
 
-try:
-    import cv2.cv as cv
-except ImportError:
-    import cv
 
 class MPSlipMapFrame(wx.Frame):
     """ The main frame of the viewer
@@ -25,24 +54,39 @@ class MPSlipMapFrame(wx.Frame):
         state.popup_started = False
         state.default_popup = None
         state.panel = MPSlipMapPanel(self, state)
+        self.last_layout_send = time.time()
         self.Bind(wx.EVT_IDLE, self.on_idle)
         self.Bind(wx.EVT_SIZE, state.panel.on_size)
-
+        self.legend_checkbox_menuitem_added = False
+        
         # create the View menu
-        self.menu = MPMenuTop([MPMenuSubMenu('View',
-                                             items=[MPMenuCheckbox('Follow\tCtrl+F', 'Follow Aircraft', 'toggleFollow',
-                                                                   checked=state.follow),
-                                                    MPMenuCheckbox('Grid\tCtrl+G', 'Enable Grid', 'toggleGrid',
-                                                                   checked=state.grid),
-                                                    MPMenuItem('Goto\tCtrl+P', 'Goto Position', 'gotoPosition'),
-                                                    MPMenuItem('Brightness +\tCtrl+B', 'Increase Brightness', 'increaseBrightness'),
-                                                    MPMenuItem('Brightness -\tCtrl+Shift+B', 'Decrease Brightness', 'decreaseBrightness'),
-                                                    MPMenuCheckbox('Download Tiles\tCtrl+D', 'Enable Tile Download', 'toggleDownload',
-                                                                   checked=state.download),
-                                                    MPMenuRadio('Service', 'Select map service',
-                                                                returnkey='setService',
-                                                                selected=state.mt.get_service(),
-                                                                items=state.mt.get_service_list())])])
+        self.menu = MPMenuTop([
+            MPMenuSubMenu('View', items=[
+                MPMenuCheckbox('Follow\tCtrl+F',
+                               'Follow Aircraft',
+                               'toggleFollow',
+                               checked=state.follow),
+                MPMenuCheckbox('Grid\tCtrl+G',
+                               'Enable Grid',
+                               'toggleGrid',
+                               checked=state.grid),
+                MPMenuItem('Goto\tCtrl+P',
+                           'Goto Position',
+                           'gotoPosition'),
+                MPMenuItem('Brightness +\tCtrl+B',
+                           'Increase Brightness',
+                           'increaseBrightness'),
+                MPMenuItem('Brightness -\tCtrl+Shift+B',
+                           'Decrease Brightness',
+                           'decreaseBrightness'),
+                MPMenuCheckbox('Download Tiles\tCtrl+D',
+                               'Enable Tile Download',
+                               'toggleDownload',
+                               checked=state.download),
+                MPMenuRadio('Service', 'Select map service',
+                            returnkey='setService',
+                            selected=state.mt.get_service(),
+                            items=state.mt.get_service_list())])])
         self.SetMenuBar(self.menu.wx_menu())
         self.Bind(wx.EVT_MENU, self.on_menu)
 
@@ -65,7 +109,7 @@ class MPSlipMapFrame(wx.Frame):
             if ret is not None:
                 ret.call_handler()
                 state.event_queue.put(SlipMenuEvent(state.popup_latlon, event, [], ret))
-            
+
         # otherwise a normal menu
         ret = self.menu.find_selected(event)
         if ret is None:
@@ -73,6 +117,8 @@ class MPSlipMapFrame(wx.Frame):
         ret.call_handler()
         if ret.returnkey == 'toggleGrid':
             state.grid = ret.IsChecked()
+        elif ret.returnkey == 'toggleLegend':
+            state.legend = ret.IsChecked()
         elif ret.returnkey == 'toggleFollow':
             state.follow = ret.IsChecked()
         elif ret.returnkey == 'toggleDownload':
@@ -82,16 +128,20 @@ class MPSlipMapFrame(wx.Frame):
         elif ret.returnkey == 'gotoPosition':
             state.panel.enter_position()
         elif ret.returnkey == 'increaseBrightness':
-            state.brightness *= 1.25
+            state.brightness += 20
+            if state.brightness > 255:
+                state.brightness = 255
         elif ret.returnkey == 'decreaseBrightness':
-            state.brightness /= 1.25
+            state.brightness -= 20
+            if state.brightness < -255:
+                state.brightness = -255
         state.need_redraw = True
 
     def find_object(self, key, layers):
         '''find an object to be modified'''
         state = self.state
 
-        if layers is None:
+        if layers is None or layers == '':
             layers = state.layers.keys()
         for layer in layers:
             if key in state.layers[layer]:
@@ -117,14 +167,26 @@ class MPSlipMapFrame(wx.Frame):
         (lat, lon) = object.latlon
         state.panel.re_center(state.width/2, state.height/2, lat, lon)
 
+    def add_legend_checkbox_menuitem(self):
+        self.menu.add_to_submenu(['View'], [MPMenuCheckbox('Legend\tCtrl+L',
+                                                         'Enable Legend',
+                                                         'toggleLegend',
+                                                         checked=self.state.legend)
+        ])
+
     def add_object(self, obj):
-        '''add an object to a later'''
+        '''add an object to a layer'''
         state = self.state
         if not obj.layer in state.layers:
             # its a new layer
             state.layers[obj.layer] = {}
         state.layers[obj.layer][obj.key] = obj
         state.need_redraw = True
+        if (not self.legend_checkbox_menuitem_added and
+            isinstance(obj, SlipFlightModeLegend)):
+            self.add_legend_checkbox_menuitem()
+            self.legend_checkbox_menuitem_added = True
+            self.SetMenuBar(self.menu.wx_menu())
 
     def remove_object(self, key):
         '''remove an object by key from all layers'''
@@ -140,12 +202,20 @@ class MPSlipMapFrame(wx.Frame):
         if state.close_window.acquire(False):
             self.state.app.ExitMainLoop()
 
+        now = time.time()
+        if now - self.last_layout_send > 1:
+            self.last_layout_send = now
+            state.event_queue.put(win_layout.get_wx_window_layout(self))
+
         # receive any display objects from the parent
         obj = None
 
         while not state.object_queue.empty():
             obj = state.object_queue.get()
 
+            if isinstance(obj, win_layout.WinLayout):
+                win_layout.set_wx_window_layout(self, obj)
+                
             if isinstance(obj, SlipObject):
                 self.add_object(obj)
 
@@ -156,6 +226,10 @@ class MPSlipMapFrame(wx.Frame):
                     object.update_position(obj)
                     if getattr(object, 'follow', False):
                         self.follow(object)
+                    if obj.label is not None:
+                        object.label = obj.label
+                    if obj.colour is not None:
+                        object.colour = obj.colour
                     state.need_redraw = True
 
             if isinstance(obj, SlipDefaultPopup):
@@ -177,6 +251,22 @@ class MPSlipMapFrame(wx.Frame):
                 state.panel.re_center(state.width/2, state.height/2, lat, lon)
                 state.need_redraw = True
 
+            if isinstance(obj, SlipZoom):
+                # change zoom
+                state.panel.set_ground_width(obj.ground_width)
+                state.need_redraw = True
+                
+            if isinstance(obj, SlipFollow):
+                # enable/disable follow
+                state.follow = obj.enable
+
+            if isinstance(obj, SlipFollowObject):
+                # enable/disable follow on an object
+                for layer in state.layers:
+                    if obj.key in state.layers[layer]:
+                        if hasattr(state.layers[layer][obj.key], 'follow'):
+                            state.layers[layer][obj.key].follow = obj.enable
+                
             if isinstance(obj, SlipBrightness):
                 # set map brightness
                 state.brightness = obj.brightness
@@ -201,7 +291,19 @@ class MPSlipMapFrame(wx.Frame):
                     if obj.key in state.layers[layer]:
                         state.layers[layer][obj.key].set_hidden(obj.hide)
                 state.need_redraw = True
-        
+
+        if state.timelim_pipe is not None:
+            while state.timelim_pipe[1].poll():
+                try:
+                    obj = state.timelim_pipe[1].recv()
+                except Exception:
+                    state.timelim_pipe = None
+                    break
+                for layer in state.layers:
+                    for key in state.layers[layer].keys():
+                        state.layers[layer][key].set_time_range(obj)
+                state.need_redraw = True
+
         if obj is None:
             time.sleep(0.05)
 
@@ -256,7 +358,7 @@ class MPSlipMapPanel(wx.Panel):
         self.mainSizer.AddSpacer(2)
 
         # panel for the main map image
-        self.imagePanel = mp_widgets.ImagePanel(self, wx.EmptyImage(state.width,state.height))
+        self.imagePanel = mp_widgets.ImagePanel(self, np.zeros((state.height, state.width, 3), dtype=np.uint8))
         self.mainSizer.Add(self.imagePanel, flag=wx.GROW, border=5)
         self.imagePanel.Bind(wx.EVT_MOUSE_EVENTS, self.on_mouse)
         self.imagePanel.Bind(wx.EVT_KEY_DOWN, self.on_key_down)
@@ -294,6 +396,12 @@ class MPSlipMapPanel(wx.Panel):
         bearing  = mp_util.gps_bearing(lat2, lon2, lat, lon)
         (state.lat, state.lon) = mp_util.gps_newpos(state.lat, state.lon, bearing, distance)
 
+    def set_ground_width(self, ground_width):
+        '''set ground width of view'''
+        state = self.state
+        state.ground_width = ground_width
+        state.panel.re_center(state.width/2, state.height/2, state.lat, state.lon)
+         
     def change_zoom(self, zoom):
         '''zoom in or out by zoom factor, keeping centered'''
         state = self.state
@@ -304,7 +412,7 @@ class MPSlipMapPanel(wx.Panel):
         (lat,lon) = self.coordinates(x, y)
         state.ground_width *= zoom
         # limit ground_width to sane values
-        state.ground_width = max(state.ground_width, 20)
+        state.ground_width = max(state.ground_width, 2)
         state.ground_width = min(state.ground_width, 20000000)
         self.re_center(x,y, lat, lon)
 
@@ -329,11 +437,11 @@ class MPSlipMapPanel(wx.Panel):
         alt = 0
         if pos is not None:
             (lat,lon) = self.coordinates(pos.x, pos.y)
-            newtext += 'Cursor: %f %f (%s)' % (lat, lon, mp_util.latlon_to_grid((lat, lon)))
+            newtext += 'Cursor: %.8f %.8f (%s)' % (lat, lon, mp_util.latlon_to_grid((lat, lon)))
             if state.elevation:
                 alt = self.ElevationMap.GetElevation(lat, lon)
                 if alt is not None:
-                    newtext += ' %.1fm' % alt
+                    newtext += ' %.1fm %uft' % (alt, alt*3.28084)
         state.mt.set_download(state.download)
         pending = 0
         if state.download:
@@ -344,7 +452,7 @@ class MPSlipMapPanel(wx.Panel):
             newtext += ' SRTM Downloading '
         newtext += '\n'
         if self.click_pos is not None:
-            newtext += 'Click: %f %f (%s %s) (%s)' % (self.click_pos[0], self.click_pos[1],
+            newtext += 'Click: %.8f %.8f (%s %s) (%s)' % (self.click_pos[0], self.click_pos[1],
                                                       mp_util.degrees_to_dms(self.click_pos[0]),
                                                       mp_util.degrees_to_dms(self.click_pos[1]),
                                                       mp_util.latlon_to_grid(self.click_pos))
@@ -353,7 +461,7 @@ class MPSlipMapPanel(wx.Panel):
                                             self.click_pos[0], self.click_pos[1])
             bearing = mp_util.gps_bearing(self.last_click_pos[0], self.last_click_pos[1],
                                             self.click_pos[0], self.click_pos[1])
-            newtext += '  Distance: %.1fm Bearing %.1f' % (distance, bearing)
+            newtext += '  Distance: %.3fm %.3fnm Bearing %.1f' % (distance, distance*0.000539957, bearing)
         if newtext != state.oldtext:
             self.position.Clear()
             self.position.WriteText(newtext)
@@ -372,10 +480,11 @@ class MPSlipMapPanel(wx.Panel):
 
     def draw_objects(self, objects, bounds, img):
         '''draw objects on the image'''
-        keys = objects.keys()
-        keys.sort()
+        keys = sorted(objects.keys())
         for k in keys:
             obj = objects[k]
+            if not self.state.legend and isinstance(obj, SlipFlightModeLegend):
+                continue
             bounds2 = obj.bounds()
             if bounds2 is None or mp_util.bounds_overlap(bounds, bounds2):
                 obj.draw(img, self.pixmapper, bounds)
@@ -384,7 +493,7 @@ class MPSlipMapPanel(wx.Panel):
         '''redraw the map with current settings'''
         state = self.state
 
-        view_same = (self.last_view and self.map_img and self.last_view == self.current_view())
+        view_same = (self.last_view is not None and self.map_img is not None and self.last_view == self.current_view())
 
         if view_same and not state.need_redraw:
             return
@@ -392,16 +501,19 @@ class MPSlipMapPanel(wx.Panel):
         # get the new map
         self.map_img = state.mt.area_to_image(state.lat, state.lon,
                                               state.width, state.height, state.ground_width)
-        if state.brightness != 1.0:
-            cv.ConvertScale(self.map_img, self.map_img, scale=state.brightness)
-
-
+        if state.brightness != 0: # valid state.brightness range is [-255, 255]
+            brightness = np.uint8(np.abs(state.brightness))
+            if state.brightness > 0:
+                self.map_img = np.where((255 - self.map_img) < brightness, 255, self.map_img + brightness)
+            else:
+                self.map_img = np.where((255 + self.map_img) < brightness, 0, self.map_img - brightness)
+            
         # find display bounding box
         (lat2,lon2) = self.coordinates(state.width-1, state.height-1)
         bounds = (lat2, state.lon, state.lat-lat2, lon2-state.lon)
 
         # get the image
-        img = cv.CloneImage(self.map_img)
+        img = self.map_img.copy()
 
         # possibly draw a grid
         if state.grid:
@@ -409,7 +521,7 @@ class MPSlipMapPanel(wx.Panel):
 
         # draw layer objects
         keys = state.layers.keys()
-        keys.sort()
+        keys = sorted(list(keys))
         for k in keys:
             self.draw_objects(state.layers[k], bounds, img)
 
@@ -418,9 +530,7 @@ class MPSlipMapPanel(wx.Panel):
             state.info[key].draw(state.panel, state.panel.information)
 
         # display the image
-        self.img = wx.EmptyImage(state.width,state.height)
-        self.img.SetData(img.tostring())
-        self.imagePanel.set_image(self.img)
+        self.imagePanel.set_image(img)
 
         self.update_position()
 
@@ -433,7 +543,6 @@ class MPSlipMapPanel(wx.Panel):
     def on_redraw_timer(self, event):
         '''the redraw timer ensures we show new map tiles as they
         are downloaded'''
-        state = self.state
         self.redraw_map()
 
     def on_size(self, event):
@@ -446,12 +555,16 @@ class MPSlipMapPanel(wx.Panel):
 
     def on_mouse_wheel(self, event):
         '''handle mouse wheel zoom changes'''
-        state = self.state
-        rotation = event.GetWheelRotation() / event.GetWheelDelta()
-        if rotation > 0:
-            zoom = 1.0/(1.1 * rotation)
-        elif rotation < 0:
-            zoom = 1.1 * (-rotation)
+        # >>> print -1/120
+        # -1
+        wheel_rotation = event.GetWheelRotation()
+        rotation = abs(wheel_rotation) // event.GetWheelDelta()
+        if rotation == 0:
+            return
+        zoom = 1.1 * rotation
+        if wheel_rotation > 0:
+            # zooming out
+            zoom = 1.0/zoom
         self.change_zoom(zoom)
         self.redraw_map()
 
@@ -475,7 +588,7 @@ class MPSlipMapPanel(wx.Panel):
         if selected.popup_menu is not None:
             import copy
             popup_menu = selected.popup_menu
-            if state.default_popup.popup is not None and state.default_popup.combine:
+            if state.default_popup is not None and state.default_popup.combine:
                 popup_menu = copy.deepcopy(popup_menu)
                 popup_menu.add(MPMenuSeparator())
                 popup_menu.combine(state.default_popup.popup)
@@ -499,7 +612,16 @@ class MPSlipMapPanel(wx.Panel):
             self.mouse_pos = pos
         self.update_position()
 
-        if event.ButtonIsDown(wx.MOUSE_BTN_ANY) or event.ButtonUp():
+        if hasattr(event, 'ButtonIsDown'):
+            any_button_down = event.ButtonIsDown(wx.MOUSE_BTN_ANY)
+            left_button_down = event.ButtonIsDown(wx.MOUSE_BTN_LEFT)
+            right_button_down = event.ButtonIsDown(wx.MOUSE_BTN_RIGHT)
+        else:
+            left_button_down = event.leftIsDown
+            right_button_down = event.rightIsDown
+            any_button_down = left_button_down or right_button_down
+
+        if any_button_down or event.ButtonUp():
             # send any event with a mouse button to the parent
             latlon = self.coordinates(pos.x, pos.y)
             selected = self.selected_objects(pos)
@@ -519,7 +641,7 @@ class MPSlipMapPanel(wx.Panel):
                     self.show_default_popup(pos)
                     state.popup_started = True
 
-        if not event.ButtonIsDown(wx.MOUSE_BTN_RIGHT):
+        if not right_button_down:
             state.popup_started = False
 
         if event.LeftDown() or event.RightDown():
@@ -527,7 +649,7 @@ class MPSlipMapPanel(wx.Panel):
             self.last_click_pos = self.click_pos
             self.click_pos = self.coordinates(pos.x, pos.y)
 
-        if event.Dragging() and event.ButtonIsDown(wx.MOUSE_BTN_LEFT):
+        if event.Dragging() and left_button_down:
             # drag map to new position
             newpos = pos
             if self.mouse_down and newpos:
@@ -562,16 +684,18 @@ class MPSlipMapPanel(wx.Panel):
             selected = self.selected_objects(self.mouse_pos)
             state.event_queue.put(SlipKeyEvent(latlon, event, selected))
 
-        c = event.GetUniChar()
+        if hasattr(event,'GetUnicodeKey'):
+            c = event.GetUnicodeKey()
+        else:
+            c = event.GetUniChar()
         if c == ord('+') or (c == ord('=') and event.ShiftDown()):
             self.change_zoom(1.0/1.2)
-            event.Skip()
         elif c == ord('-'):
             self.change_zoom(1.2)
-            event.Skip()
-        elif c == ord('G'):
+        elif c == ord('G') and not event.ControlDown():
             self.enter_position()
-            event.Skip()
         elif c == ord('C'):
             self.clear_thumbnails()
+        else:
+            # propogate event:
             event.Skip()
